@@ -500,28 +500,68 @@ func TestOverrideLookupLimit(t *testing.T) {
 	dns.Txt["domain3"] = []string{"v=spf1 include:domain4"}
 	dns.Txt["domain4"] = []string{"v=spf1 +all"}
 
-	// The default of 10 should be enough.
+	// The default of 10 should be plenty enough.
 	res, err := CheckHostWithSender(ip1111, "helo", "user@domain1")
 	if res != Pass {
 		t.Errorf("expected pass, got %q / %q", res, err)
 	}
 
-	// Set the limit to 4, which is enough.
+	// Set the limit to 3, which is just enough.
 	res, err = CheckHostWithSender(ip1111, "helo", "user@domain1",
-		OverrideLookupLimit(4))
+		OverrideLookupLimit(3))
 	if res != Pass {
 		t.Errorf("expected pass, got %q / %q", res, err)
 	}
 
-	// Set the limit to 3, which is not enough.
+	// Set the limit to 2, which is not enough.
 	res, err = CheckHostWithSender(ip1111, "helo", "user@domain1",
-		OverrideLookupLimit(3))
+		OverrideLookupLimit(2))
 	if res != PermError || err != ErrLookupLimitReached {
 		t.Errorf("expected permerror/lookup limit reached, got %q / %q",
 			res, err)
 	}
 }
 
+func TestOverrideVoidLookupLimit(t *testing.T) {
+	dns := NewDefaultResolver()
+	defaultTrace = t.Logf
+
+	nxDomainErr := &net.DNSError{
+		Err:         "no such domain",
+		IsTemporary: false,
+		IsNotFound:  true,
+	}
+
+	dns.Txt["domain1"] = []string{"v=spf1 exists:%{i}.one include:domain2"}
+	dns.Txt["domain2"] = []string{"v=spf1 exists:%{i}.two include:domain3"}
+	dns.Txt["domain3"] = []string{"v=spf1 exists:%{i}.three include:domain4"}
+	dns.Txt["domain4"] = []string{"v=spf1 +all"}
+	dns.Errors["1.1.1.1.one"] = nxDomainErr
+	dns.Errors["1.1.1.1.two"] = nxDomainErr
+	dns.Errors["1.1.1.1.three"] = nxDomainErr
+
+	// The default of 2
+	res, err := CheckHostWithSender(ip1111, "helo", "user@domain1")
+	if res != PermError {
+		t.Errorf("expected permerror, got %q / %q", res, err)
+	}
+
+	// Set the limit to 10, which is excessive.
+	res, err = CheckHostWithSender(ip1111, "helo", "user@domain1",
+		OverrideVoidLookupLimit(10))
+	if res != Pass {
+		t.Errorf("expected pass, got %q / %q", res, err)
+	}
+
+	// Set the limit to 1, which is not enough.
+	res, err = CheckHostWithSender(ip1111, "helo", "user@domain1",
+		OverrideVoidLookupLimit(1))
+	if res != PermError || err != ErrVoidLookupLimitReached {
+		t.Errorf("expected permerror/void lookup limit reached, got %q / %q",
+			res, err)
+	}
+}
+
 func TestWithContext(t *testing.T) {
 	dns := NewDefaultResolver()
 	defaultTrace = t.Logf
